package net.sit.conf;

/**
 * @author Christoph Sandhaus
 *         <p>
 *         Speichern von Applikationseigenschaften in einer Konfigurationsdatei.
 *         <p>
 *         Die <code>AppSettings</code> sind als Singelton implementiert,
 *         damit pro Applikation nur eine Instanz auf die Konfigurationsdatei
 *         zugreift.
 *         <p>
 *         WICHTIG: <br>
 *         Beim ersten Aufruf <i>mu </i> ein Name der Applikation angegeben
 *         werden. Dieser Name wird im Pfad und Dateinamen der
 *         Konfigurationsdatei verwendet. <br>
 *         Wird die Methode <code>getInstance()</code> erstmalig ohne
 *         Parameter aufgerufen, so gibt die Methode <code>null</code> zurck.
 *         <p>
 *         Beispiel: <br>
 *         Erstmaliges instanziieren der <code>AppSettings</code>:<br>
 *         <code>AppSettings cfg = AppSettings.Factory.getInstance("MyProgName");
 *         </code>
 *         <p>
 *         Es wird versucht folgende Datei anzulegen: <br>
 *         <code>$HOME/.MyProgName/MyProgName.config</code>
 *         <p>
 *         Im Folgenden kann der parameterlose Konstruktor verwendet werden.
 *         <p>
 *         Der Zugriff auf die Informationen geschieht in gleicher Weise wie in
 *         der Klasse <code>Properties</code>.
 * 
 */
public interface AppSettings {
	/**
	 * Schreiben der Konfigurationsdaten in die Datei.
	 * 
	 * @return true bei Erfolg, sonst false
	 */
	public boolean storeSettings();

	/**
	 * Searches for the property with the specified key in this property list.
	 * If the key is not found in this property list, the default property list,
	 * and its defaults, recursively, are then checked. The method returns null
	 * if the property is not found.
	 * 
	 * @param key
	 *            the property key
	 * @return the value in this property list with the specified key value.
	 */
	
	public String getProperty(String key);

	/**
	 * Searches for the property with the specified key in this property list.
	 * If the key is not found in this property list, the default property list,
	 * and its defaults, recursively, are then checked. The method returns the
	 * default value argument if the property is not found.
	 * 
	 * @param key
	 *            the property key
	 * @param defaultValue
	 *            a default value
	 * @return the value in this property list with the specified key value.
	 */
	public String getProperty(String key, String defaultValue);

	/**
	 * Calls the <tt>Hashtable</tt> method <code>put</code>. Provided for
	 * parallelism with the <tt>getProperty</tt> method. Enforces use of
	 * strings for property keys and values. The value returned is the result of
	 * the <tt>Hashtable</tt> call to <code>put</code>.
	 * 
	 * @param key
	 *            the key to be placed into this property list
	 * @param value
	 *            the value corresponding to key
	 */
	public void setProperty(String key, String value);

	/**
	 * user's home directory
	 * 
	 * @return user's home directory
	 */
	public String getUserHome();

	/**
	 * system file separator
	 * 
	 * @return system file separator
	 */
	public String getFileSeparator();

	/**
	 * Identify if config file was newly created on applications startup.
	 * 
	 * @return true if config file war newly created, false otherwise
	 */
	public boolean isNewConfig();

	/**
	 * @author Christoph Sandhaus
	 */
	public class Factory {

		/**
		 * Instantiating the Interface AppSettings the config directory for this
		 * application is searched, if missing, it will be created. The path
		 * will be constructed: <br>- the user's home directory <br>- a
		 * leading dot <br>- the name of the application
		 * 
		 * @param appName
		 *            a short descriptive name of the application
		 * @return instance of AppSettings
		 */
		public static AppSettings getInstance(String appName) {
			// return ApplicationSettingsImpl.getInstance(configFile);
			return ApplicationSettingsImpl.getInstance(appName);
		}

		public static AppSettings getInstance() {
			// return ApplicationSettingsImpl.getInstance(configFile);
			return ApplicationSettingsImpl.getInstance();
		}

	}
}