package net.sit.conf;

import org.apache.log4j.Logger;

import java.util.Properties;
import java.io.*;

/**
 * User: Christoph Sandhaus
 */

public class ApplicationSettingsImpl implements AppSettings {
	static Logger logger = Logger.getLogger(AppSettings.class
			.getName());

	private static ApplicationSettingsImpl applicSettings;

	private Properties mySettings;

	private boolean newConfig;

	private final String fs = System.getProperty("file.separator");

	private final String userHomePath = System.getProperty("user.home");

	private final String confExtension = ".config";

	private File configFile;

	private ApplicationSettingsImpl(String appName) {
		// construct config file name
		configFile = new File(userHomePath + fs + "." + appName + fs + appName
				+ confExtension);
		mySettings = new Properties();

		if (!configFile.canRead()) {
			logger.debug("Config file '" + configFile.getPath()
					+ "' is not readable.");
			// create default config
			File configDir = new File(userHomePath + fs + "." + appName);
			if (!configDir.exists()) {
				logger.debug("Creating project directory '"
						+ configDir.getName() + "'");
				configDir.mkdirs();
			}
			try {
				configFile.createNewFile();
				logger.debug("Config file '" + configFile.getPath()
						+ "' created.");
				this.newConfig = true;
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			FileInputStream in = null;
			try {
				in = new FileInputStream(configFile);
				mySettings.load(in);
				in.close();
				this.newConfig = false;
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Abholen der vorhandenen Instanz, ggf. erzeugen.
	 * @param appName Der Name der Applikation (wird im Dateipfad verwendet)
	 * @return Instanz von ApplicationSettingsImpl, bei Fehler null
	 * <p>
	 * Wenn die Instanz der Konfiguration erzeugt werden soll ist der Name
	 * der Applikation notwendig, da dieser Name im Pfad zur Konfiguarionsdatei
	 * verwendet wird.
	 * <p>
	 * Beilspiel:
	 * <br><code>getInstance("browser")</code>
	 * <br>erzeugt einen Konfigurtionspfad:
	 * <br><code>$HOME/.browser/browser.config</code>
	 */
	public synchronized static ApplicationSettingsImpl getInstance(
			String appName) {
		
		if (appName == null)
			return null;
		
		if (applicSettings == null) {
			applicSettings = new ApplicationSettingsImpl(appName);
		}
		return applicSettings;
	}

	/**
	 * Die vorhandene Instanz der Konfiguration holen
	 * 
	 * @return falls vorhanden: die Instanz, sonst null
	 *         <p>
	 *         Um eine Instanz der Konfiguration zu erstellen ist es beim ersten
	 *         Aufruf erfordertlich, einen Namen der Applikation mitzuteilen.
	 *         Dieser Name wird dann in dem Pfad zur Konfigurationsdatei
	 *         verwendet.
	 *         <p>
	 *         Wird <code>getInstance()</code> aufgerufen, bevor
	 *         <code>getInstance(AppName)</code> das erste mal aufgerufen
	 *         wurde, kann kein eindeutiger Pfad zur Konfigurationsdatei
	 *         aufgebaut werden und es wird null zurckgegeben.
	 */
	public synchronized static ApplicationSettingsImpl getInstance() {
		if (applicSettings == null)
			return null;
		else
			return applicSettings;
	}

	/* (non-Javadoc)
	 * @see net.sit.conf.AppSettings#storeSettings()
	 */
	public boolean storeSettings() {
		FileOutputStream out = null;
		try {
			out = new FileOutputStream(configFile);
			mySettings.store(out,
					"Automatic generated config file: DO NOT EDIT!");

			out.close();

			this.newConfig = false;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}

		return true;
	}

	public String getProperty(String key) {
		return mySettings.getProperty(key);
	}

	public String getProperty(String key, String defaultValue) {
		// if property does not exist, store default value
		if (mySettings.getProperty(key) == null) {
			mySettings.setProperty(key, defaultValue);
			storeSettings();
		}

		return mySettings.getProperty(key, defaultValue);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sit.conf.AppSettings#setProperty(java.lang.String,
	 *      java.lang.String)
	 */
	public void setProperty(String key, String value) {
		mySettings.setProperty(key, value);
	}

	public String getUserHome() {
		return userHomePath;
	}

	public String getFileSeparator() {
		return fs;
	}

	public boolean isNewConfig() {
		return newConfig;
	}

	public boolean configExists() {
		return true;
	}
}